#include "dex.h"
#include "ui_dex.h"

dex::dex(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::dex)
{
    ui->setupUi(this);
}

dex::~dex()
{
    delete ui;
}


/****************************************************************************************************************************
 *  hexNumberChanged is called when what is in the hexLineEdit in the UI is altered. The string of what is in that          *
 * QLineEdit is passed in the form of a QString. Check the signals/slots in hex.ui to see how this fits together.           *
 * Every time a new character is typed, it is automatically converted to upper case. Then the string is parsed to ensure    *
 * there are no characters that are not part of a hex value. ie outside of 0-F.                                             *
 * The decimal value is written to the decimal value's QLineEdit                                                            *
 ***************************************************************************************************************************/
void dex::hexNumberChanged(QString hexString)
{

    quint64 hexNumber;
    bool ok;
    QChar c;
    char i;
    quint8 len;
    quint8 k;
    quint8 pos;

// First set any value that is typed to upper case.
    pos = ui->hexLineEdit->cursorPosition();
    c = hexString[pos-1];
    hexString = hexString.toUpper();

    len = hexString.length();

    k = 0;

// Now parse the string to ensure there are no non-hexadecimal values.
    while (k < len)
        {
        c = hexString[k];
        i = c.toLatin1();
        if ((i < 48) || ((i > 57) && (i < 65)) || (i > 71))
            {
            hexString = hexString.remove(k,1);
            ui->hexLineEdit->setText(hexString);
            pos--;
            k = len + 1;
            }
        k++;
        }

// Convert the string to a hex number
    hexNumber = hexString.toInt(&ok,16);
// Write hte corrected string back into the QLineEdit
    ui->hexLineEdit->setText(hexString);
// Then write the decimal value into the decimal box
    ui->decimaLineEdit->setText(QString::number(hexNumber,10));
// Finally set the cursor position back to the end of the line
    ui->hexLineEdit->setCursorPosition(pos);

}



/****************************************************************************************************************************
 *  decimalNumberChanged is called when what is in the decimalLineEdit in the UI is altered. The string of what is in that  *
 * QLineEdit is passed in the form of a QString. Check the signals/slots in hex.ui to see how this fits together.           *
 * The the string is parsed to ensure  there are no characters that are not part of a decimal value. ie outside of 0-9      *
 * The hex value is written to the hex value's QLineEdit                                                                    *
 ***************************************************************************************************************************/
void dex::decimalNumberChanged(QString decimalString)
{

    quint64 decimalNumber;
    QChar c;
    char i;
    quint8 len;
    quint8 k;
    quint8 pos;


    len = decimalString.length();
    pos = ui->decimaLineEdit->cursorPosition();
    k = 0;
// Go through and check nothing in the string is outside of the value 0-9 inclusive
    while (k < len)
        {
        c = decimalString[k];
        i = c.toLatin1();

        if ((i < 48) || (i > 57))
            {
            decimalString = decimalString.remove(k,1);
            ui->decimaLineEdit->setText(decimalString);
            pos--;
            k = len + 1;
            }
        k++;
        }
// Turn the string into a number
    decimalNumber = decimalString.toLong();
// turn the number into a hex string and write to the hex line edit
    ui->hexLineEdit->setText(QString::number(decimalNumber,16).toUpper());
// set the cursor back to the end of the decimal string in the line edit
    ui->decimaLineEdit->setCursorPosition(pos);

}

